@@ -1,18 +1,49 @@
 package Outthentic::DSL;
 
 use strict;
-require Test::More;
 
-our $VERSION = '0.0.2';
+our $VERSION = '0.0.3';
+
+use Carp;
+use Data::Dumper;
+$Data::Dumper::Terse=1;
+
+sub results {
+
+    my $self = shift;
+
+    $self->{results};
+}
+
+sub add_result {
+
+    my $self = shift;
+    my $item = shift;
+
+    push @{$self->results}, { %{$item}, type => 'check_expression' };
+        
+}
+
+sub add_debug_result {
+
+    my $self = shift;
+    my $item = shift;
+
+    push @{$self->results}, { message => $item , type => 'debug' };
+        
+}
+
 
 sub new {
 
     my $class = shift;
-    my $opts = shift;
+    my $output = shift;
+    my $opts = shift || {};
 
     bless {
-        context => [],
-        context_local => [],
+        results => [],
+        output_context => [],
+        search_context => [],
         context_populated => 0,
         captures => [],
         within_mode => 0,
@@ -20,9 +51,9 @@ sub new {
         last_match_line => undef,
         last_check_status => undef,
         debug_mod => 0,
-        output => undef,
+        output => $output||'',
         match_l => 40,
-        $opts,
+        %{$opts},
     }, __PACKAGE__;
 
 }
@@ -35,19 +66,19 @@ sub populate_context {
 
     my $i = 0;
 
-    my @context = ();
+    my @output_context = ();
 
     for my $l ( split /\n/, $self->{output} ){
         chomp $l;
         $i++;
         $l=":blank_line" unless $l=~/\S/;
-        push @context, [$l, $i];
+        push @output_context, [$l, $i];
     }
 
-    $self->{context} = [@context];
-    $self->{context_local} = [@context];
+    $self->{output_context} = [@output_context];
+    $self->{search_context} = [@output_context];
 
-    Test::More::diag("context populated") if $self->{debug_mod} >= 2;
+    $self->add_debug_result('context populated') if $self->{debug_mod} >= 2;
 
     $self->{context_populated} = 1;
 
@@ -73,27 +104,27 @@ sub check_line {
 
     $self->reset_captures;
 
-    my @captures;
+    my @captures = ();
 
     $self->populate_context;
 
-    Test::More::diag("lookup $pattern ...") if $self->{debug_mod} >= 2;
+    $self->add_debug_result("lookup $pattern ...") if $self->{debug_mod} >= 2;
 
-    my @context         = @{$self->{context}};
-    my @context_local   = @{$self->{context_local}};
+    my @output_context         = @{$self->{output_context}};
+    my @search_context   = @{$self->{search_context}};
     my @context_new     = ();
 
     if ($check_type eq 'default'){
-        for my $c (@context_local){
+        for my $c (@search_context){
             my $ln = $c->[0]; my $next_i = $c->[1];
             if ( index($ln,$pattern) != -1){
                 $status = 1;
                 $self->{last_match_line} = $ln;
             }
-            push @context_new, $context[$next_i] if $self->{block_mode};
+            push @context_new, $output_context[$next_i] if $self->{block_mode};
         }
     }elsif($check_type eq 'regexp'){
-        for my $c (@context_local){
+        for my $c (@search_context){
             my $re = qr/$pattern/;
             my $ln = $c->[0]; my $next_i = $c->[1];
 
@@ -105,41 +136,40 @@ sub check_line {
                 push @context_new, $c if $self->{within_mode};
                 $self->{last_match_line} = $ln;
             }
-            push @context_new, $context[$next_i] if $self->{block_mode};
+            push @context_new, $output_context[$next_i] if $self->{block_mode};
 
         }
     }else {
-        die "unknown check_type: $check_type";
+        confess "unknown check_type: $check_type";
     }
 
-    Test::More::ok($status,$message);
+
+
     $self->{last_check_status} = $status;
 
     if ( $self->{debug_mod} >= 2 ){
-        my $k=0;
-        for my $ce (@captures) {
-            $k++;
-            Test::More::diag("captured item N $k");
-            for  my $c (@{$ce}){
-                Test::More::diag("\tcaptures: $c");
-            }
-        }
+        $self->add_debug_result('captures:');
+        $self->add_debug_result(Dumper(\@captures));
     }
 
     $self->{captures} = [ @captures ];
 
     # update context
     if ( $self->{block_mode} ){
-        $self->{context_local} = [@context_new];
+        $self->{search_context} = [@context_new];
     } elsif ( $self->{within_mode} and $status ){
-        $self->{context_local} = [@context_new];
+        $self->{search_context} = [@context_new];
+    }elsif ( $self->{within_mode} and ! $status ){
+        $self->{search_context} = []; # empty context if within expression has not passed 
     }
 
+    $self->add_result({ status => $status , message => $message });
+
     return $status;
 
 }
 
-sub generate_asserts {
+sub validate {
 
     my $self = shift;
 
@@ -153,7 +183,7 @@ sub generate_asserts {
         @lines = @$filepath_or_array_ref
     }else{
         return unless $filepath_or_array_ref;
-        open my $fh, $filepath_or_array_ref or die $!;
+        open my $fh, $filepath_or_array_ref or confess $!;
         while (my $l = <$fh>){
             push @lines, $l
         }
@@ -165,14 +195,14 @@ sub generate_asserts {
 
         chomp $l;
 
-        Test::More::diag $l if $self->{debug_mod} >= 3;
+        $self->add_debug_result($l) if $self->{debug_mod} >= 3;
 
         next LINE unless $l =~ /\S/; # skip blank lines
 
         next LINE if $l=~ /^\s*#(.*)/; # skip comments
 
         if ($l=~ /^\s*begin:\s*$/) { # begin of text block
-            Test::More::diag("begin text block") if $self->{debug_mod} >= 2;
+            $self->add_debug_result('begin text block') if $self->{debug_mod} >= 2;
             $self->{block_mode} = 1;
             next LINE;
         }
@@ -181,16 +211,16 @@ sub generate_asserts {
             $self->{block_mode} = 0;
 
             # restore local context
-            $self->{context_local} = $self->{context};
+            $self->{search_context} = $self->{output_context};
 
-            Test::More::diag("end text block") if $self->{debug_mod} >= 2;
+            $self->add_debug_result('end text block') if $self->{debug_mod} >= 2;
 
             next LINE;
         }
 
         # validate unterminated multiline chunks
-        if ($l=~/^\s*(regexp|code|generator|within):\s*.*/){
-            die "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
+        if ($l=~/^\s*(regexp|code|generator|within|validator):\s*.*/){
+            confess "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
         }
 
         if ($l=~/^\s*code:\s*(.*)/){ # `code' line
@@ -205,6 +235,19 @@ sub generate_asserts {
                 $self->handle_code($code);
             }
 
+        }elsif($l=~/^\s*validator:\s*(.*)/){ # `validator' line
+
+            my $code = $1;
+
+            if ($code=~s/\\\s*$//){
+                 push @multiline_chunk, $code;
+                 $chunk_type = 'validator';
+                 next LINE; # this is multiline chunk, accumulate lines until meet '\' line
+
+            }else{
+                $self->handle_validator($code);
+            }
+
         }elsif($l=~/^\s*generator:\s*(.*)/){ # `generator' line
 
             my $code = $1;
@@ -257,7 +300,7 @@ sub generate_asserts {
         }
     }
 
-    die "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
+    confess "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
 
 }
 
@@ -268,13 +311,35 @@ sub handle_code {
 
     unless (ref $code){
         eval "package main; $code;";
-        die "code LINE eval perl error, code:$code , error: $@" if $@;
-        Test::More::diag("handle_code OK. $code") if $self->{debug_mod} >= 3;
+        confess "eval error; sub:handle_code; code:$code; error: $@" if $@;
+        $self->add_debug_result("handle_code OK. $code") if $self->{debug_mod} >= 3;
     } else {
         my $code_to_eval = join "\n", @$code;
         eval "package main; $code_to_eval";
-        die "code LINE eval error, code:$code_to_eval , error: $@" if $@;
-        Test::More::diag("handle_code OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+        confess "eval error; sub:handle_code; code:$code_to_eval; error: $@" if $@;
+        $self->add_debug_result("handle_code OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+    }
+
+}
+
+sub handle_validator {
+
+    my $self = shift;
+    my $code = shift;
+
+    unless (ref $code){
+        my $r = eval "package main; $code;";
+        confess "eval error; sub:handle_validator; code:$code; error: $@" if $@;
+        confess "not valid return from validator, should be ARRAYREF. got: @{[ref $r]}" unless ref($r) eq 'ARRAY' ;
+        $self->add_result({ status => $r->[0] , message => $r->[1] });
+        $self->add_debug_result("handle_validator OK. $code") if $self->{debug_mod} >= 3;
+    } else {
+        my $code_to_eval = join "\n", @$code;
+        my $r = eval "package main; $code_to_eval";
+        confess "eval error; sub:handle_validator; code:$code_to_eval; error: $@" if $@;
+        confess "not valid return from validator, should be ARRAYREF. got: @{[ref $r]}" unless ref($r) eq 'ARRAY' ;
+        $self->add_result({ status => $r->[0] , message => $r->[1] });
+        $self->add_debug_result("handle_validator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
     }
 
 }
@@ -286,15 +351,17 @@ sub handle_generator {
 
     unless (ref $code){
         my $arr_ref = eval "package main; $code";
-        die "generator LINE eval error, code:$code , error: $@" if $@;
-        Test::More::diag("handle_generator OK. $code") if $self->{debug_mod} >= 3;
-        $self->generate_asserts($arr_ref);
+        confess "eval error; sub:handle_generator; code:$code; error: $@" if $@;
+        confess "not valid return from generator, should be ARRAYREF. got: @{[ref $arr_ref]}" unless ref($arr_ref) eq 'ARRAY' ;
+        $self->add_debug_result("handle_generator OK. $code") if $self->{debug_mod} >= 3;
+        $self->validate($arr_ref);
     } else {
         my $code_to_eval = join "\n", @$code;
         my $arr_ref = eval " package main; $code_to_eval";
-        die "generator LINE eval error, code:$code_to_eval , error: $@" if $@;
-        Test::More::diag("handle_generator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
-        $self->generate_asserts($arr_ref);
+        confess "eval error; sub:handle_generator; code:$code_to_eval; error: $@" if $@;
+        confess "not valid return from generator, should be ARRAYREF. got: @{[ref $arr_ref]}" unless ref($arr_ref) eq 'ARRAY' ;
+        $self->add_debug_result("handle_generator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+        $self->validate($arr_ref);
     }
 
 }
@@ -306,9 +373,13 @@ sub handle_regexp {
     
     my $m;
 
+    my $reset_search_context = 0;
+
     if ($self->{within_mode}){
+
         $self->{within_mode} = 0; 
-        $self->{context_local} = $self->{context};
+        $reset_search_context = 1;
+
         if ($self->{last_check_status}){
             my $lml =  $self->_short_string($self->{last_match_line});
             $m = "'$lml' match /$re/";
@@ -316,13 +387,17 @@ sub handle_regexp {
             $m = "output match /$re/";
         }
     } else {
-        $m = "output @{[$self->{block_mode} ? 'block' : '' ]} match /$re/";
+        $m = "output match /$re/";
+        $m = "[b] $m" if $self->{block_mode};
     }
 
 
     $self->check_line($re, 'regexp', $m);
 
-    Test::More::diag("handle_regexp OK. $re") if $self->{debug_mod} >= 3;
+    $self->{search_context} = $self->{output_context} if $reset_search_context; 
+
+    $self->add_debug_result("handle_regexp OK. $re") if $self->{debug_mod} >= 3;
+
 
 }
 
@@ -348,7 +423,7 @@ sub handle_within {
 
     $self->check_line($re, 'regexp', $m);
 
-    Test::More::diag "handle_within OK. $re" if $self->{debug_mod} >= 3;
+    $self->add_debug_result("handle_within OK. $re") if $self->{debug_mod} >= 3;
     
 }
 
@@ -359,10 +434,13 @@ sub handle_plain {
 
     my $m;
     my $lshort =  $self->_short_string($l);
+    my $reset_search_context = 0;
 
     if ($self->{within_mode}){
+        
         $self->{within_mode} = 0;
-        $self->{context_local} = $self->{context}; 
+        $reset_search_context = 1;
+
         if ($self->{last_check_status}){
             my $lml =  $self->_short_string($self->{last_match_line});
             $m = "'$lml' match '$lshort'";
@@ -370,13 +448,16 @@ sub handle_plain {
             $m = "output match '$lshort'";
         }
     }else{
-        $m = "output @{[$self->{block_mode} ? 'block' : '' ]} match '$lshort'";
+        $m = "output match '$lshort'";
+        $m = "[b] $m" if $self->{block_mode};
     }
 
 
     $self->check_line($l, 'default', $m);
 
-    Test::More::diag("handle_plain OK. $l") if $self->{debug_mod} >= 3;
+    $self->{search_context} = $self->{output_context} if $reset_search_context; 
+
+    $self->add_debug_result("handle_plain OK. $l") if $self->{debug_mod} >= 3;
 }
 
 
@@ -387,7 +468,7 @@ sub _short_string {
     my $sstr = substr( $str, 0, $self->{match_l} );
 
     
-    return $sstr < $str ? "$str ..." : $str; 
+    return $sstr < $str ? "$sstr ..." : $sstr; 
 
 }
 
@@ -400,22 +481,24 @@ __END__
 
 =head1 SYNOPSIS
 
-Outthentic DSL
+Language to validate text output.
 
 
-=head1 Outthentic DSL
+=head1 Install
+
+    cpanm Outthentic
+
+
+=head1 Glossary
+
+
+=head2 Outthentic DSL 
 
 =over
 
 =item *
 
-DSL provides some meta language to validate I<arbitrary> plain text.
-
-
-
-=item *
-
-One should create a so called `check files' - DSL scripts to describe validation process.
+Is a language to validate I<arbitrary> plain text. Very often a short form `DSL' is used for this term. 
 
 
 
@@ -425,24 +508,28 @@ Outthentic DSL is both imperative and declarative language.
 
 
 
-=item *
-
-It's convenient to refer to the text validate by as `stdout', thinking that one program generates and yields
-some output into stdout.
-
-
-
 =back
 
 
-=head1 Check files
+=head2 Check files
 
-Check file is a regular file in text plain format. The content of check file is a DSL script.
+A plain text files containing program code written on DSL to describe text L<validation process|#validation-process>.
 
 
-=head1 Parser
+=head2 Code
 
-`Parser' is the program which:
+Content of check file. Should be program code written on DSL.
+
+
+=head2 Stdout
+
+It's convenient to refer to the text validate by as stdout, thinking that one program generates and yields output into stdout
+which is then validated.
+
+
+=head2 Parser
+
+Parser is the program which:
 
 =over
 
@@ -454,31 +541,31 @@ parses check file line by line
 
 =item *
 
-creates and then I<executes> outthentic entry represented by parsed line(s)
+creates and then I<executes> outthentic entries represented by parsed lines
 
 
 
 =item *
 
-execution of entry results in one of three things:
+execution of each entry results in one of three things:
 
 =over
 
 =item *
 
-validation stdout against check expression - if entry is check expression one
+performing L<validation|#validation> process if entry is check expression one
 
 
 
 =item *
 
-generating new outhentic entries - if entry is generator one
+generating new outthentic entries if entry is generator one
 
 
 
 =item *
 
-execution of perl code - if entry is perl expression one
+execution of perl code if entry is perl expression one
 
 
 
@@ -489,15 +576,193 @@ execution of perl code - if entry is perl expression one
 =back
 
 
-=head1 Outthentic entries
+=head2 Validation process
 
-Outhentic DSL comprises following basic entities, listed at pretty arbitrary order:
+Validation process consists of: 
 
 =over
 
 =item *
 
-check expressions:
+checking if stdout matches the check expression or
+
+
+
+=item *
+
+in case of L<validator expression|#validators> :
+
+=over
+
+=item *
+
+executing validator code and checking if returned value is true 
+
+
+=back
+
+
+
+=item *
+
+generating validation results could be retrieved later
+
+
+
+=item *
+
+a final I<presentation> of validation results should be implimeted in a certain L<client|#clients> I<using> L<parser api|#parser-api> and not being defined at DSL scope. For the sake of readability a table like form ( which is a fake one ) is used for validation results in this document. 
+
+
+
+=back
+
+
+=head2 Parser journal
+
+Parser activity could be logged into journal, see L<parser api|parser-api>, `journal' method description
+
+
+=head2 Parser API
+
+Outthentic provides program api for parser:
+
+    use Test::More qw{no_plan};
+    
+    use Outthentic::DSL;
+    
+    my $outh = Outthentic::DSL->new('stdout string', $opts);
+    $outh->validate('path/to/check/file','stdout string');
+    
+    
+    for my $r ( @{$outh->results}){
+        ok($r->{status}, $r->{message}) if $r->{type} eq 'check_expression';
+        diag($r->{message}) if $r->{type} eq 'debug';
+    
+    }
+
+Methods list:
+
+
+=head3 new
+
+This is constructor, create Outthentic::DSL instance. 
+
+Obligatory parameters is:
+
+=over
+
+=item *
+
+stdout string 
+
+
+=back
+
+Optional parameters is passed as hashref:
+
+=over
+
+=item *
+
+matchI<l - truncate matching strings to {match>l} bytes
+
+
+=back
+
+Default value is `40'
+
+=over
+
+=item *
+
+debug_mod - enable debug mode
+
+=over
+
+=item *
+
+Possible values is 0,1,2,3.
+
+
+
+=item *
+
+Set to 1 or 2 or 3 if you want to see some debug information in validation results.
+
+
+
+=item *
+
+Increasing debug_mod value means more low level information appeared.
+
+
+
+=item *
+
+Default value is `0' - means do not create debug messages.
+
+
+
+=back
+
+
+
+=back
+
+
+=head3 validate
+
+Runs parser for check file and and initiates validation process against stdout.
+
+Obligatory parameter is:
+
+=over
+
+=item *
+
+path to check file
+
+
+=back
+
+
+=head3 results
+
+
+Returns validation results as arrayref containing { type, status, message } hashrefs.
+
+
+=head2 Outthentic client
+
+Client is a external program using DSL API. Existed outthentic clients:
+
+=over
+
+=item *
+
+L<swat|https://github.com/melezhik/swat>
+
+
+=item *
+
+L<outthentic|https://github.com/melezhik/outthentic>
+
+
+=back
+
+More clients wanted :) , please L<write me|mailto:melezhik@gmail.com> if you have one!
+
+
+=head1 Outthentic entities
+
+Outthentic DSL comprises following basic entities:
+
+=over
+
+=item *
+
+Check expressions:
 
 =over
 
@@ -521,31 +786,36 @@ text blocks
 within expressions
 
 
+=item *
+
+validator expressions
+
+
 =back
 
 
 
 =item *
 
-comments
+Comments
 
 
 
 =item *
 
-blank lines
+Blank lines
 
 
 
 =item *
 
-perl expressions
+Perl expressions
 
 
 
 =item *
 
-generators
+Generator expressions
 
 
 
@@ -554,29 +824,36 @@ generators
 
 =head1 Check expressions
 
-Check expressions defines I<what lines stdout should have>:
+Check expressions defines I<lines stdout should match>. Here is a simple example:
 
     # stdout
+    
     HELLO
     HELLO WORLD
     My birth day is: 1977-04-16
     
     
     # check list
+    
     HELLO
     regexp: \d\d\d\d-\d\d-\d\d
     
     
-    # validator output
-    OK - output matches "HELLO"
-    OK - output matches /\d\d\d\d-\d\d-\d\d/
+    # validation results
+    
+    +--------+------------------------------+
+    | status | message                      |
+    +--------+------------------------------+
+    | OK     | matches "HELLO"              |
+    | OK     | matches /\d\d\d\d-\d\d-\d\d/ |
+    +--------+------------------------------+
 
-There are two type of check expressions - L<plain strings|#plain-strings> and L<regular expressions|#regular-expressions>.
+There are two basic types of check expressions - L<plain strings|#plain-strings> and L<regular expressions|#regular-expressions>.
 
 It is convenient to talk about I<check list> as of all check expressions in a given check file.
 
 
-=head1 plain string
+=head1 Plain string expressions 
 
         I am ok
         HELLO Outthentic
@@ -584,7 +861,7 @@ It is convenient to talk about I<check list> as of all check expressions in a gi
 The code above declares that stdout should have lines 'I am ok' and 'HELLO Outthentic'.
 
 
-=head1 regular expressions
+=head1 Regular expressions
 
 Similarly to plain strings matching, you may require that stdout has lines matching the regular expressions:
 
@@ -595,7 +872,7 @@ Similarly to plain strings matching, you may require that stdout has lines match
 Regular expressions should start with `regexp:' marker.
 
 
-=head1 captures
+=head1 One or many?
 
 Parser does not care about I<how many times> a given check expression is found in stdout.
 
@@ -612,14 +889,8 @@ See L<"captures"|#captures> section for full explanation of a captures mechanism
 
 Comments and blank lines don't impact validation process but one could use them to improve code readability.
 
-=over
 
-=item *
-
-B<comments>
-
-
-=back
+=head2 Comments
 
 Comment lines start with `#' symbol, comments chunks are ignored by parser:
 
@@ -627,14 +898,8 @@ Comment lines start with `#' symbol, comments chunks are ignored by parser:
     The beginning of story
     Hello World # or could be added to existed expression to the right, like here
 
-=over
 
-=item *
-
-B<blank lines>
-
-
-=back
+=head2 Blank lines
 
 Blank lines are ignored as well:
 
@@ -658,14 +923,8 @@ But you B<can't ignore> blank lines in a `text block matching' context ( see `te
         :blank_line
     end:
 
-=over
 
-=item *
-
-B<text blocks>
-
-
-=back
+=head2 Text blocks
 
 Sometimes it is very helpful to match a stdout against a `block of strings' goes consequentially, like here:
 
@@ -710,6 +969,7 @@ Parser will remain in a `text block' mode till the end of check file, which is p
         begin:
             here we begin
             and till the very end of test
+    
             we are in `text block` mode
 
 
@@ -724,44 +984,14 @@ Perl expressions are just a pieces of perl code to I<get evaled> during parsing
 
 Internally once check file gets parsed this piece of DSL code is "turned" into regular perl code:
 
-    ok($status,"stdout matches Once upon a time");
+    execute_check_expression("Once upon a time");
     eval 'print "Lived a boy called Outthentic"';
-    ok($status,"stdout matches Lived a boy called Outthentic");
+    execute_check_expression("Lived a boy called Outthentic");
 
-So, all perl expressions in DSL code will be replaced by perl eval {code} expressions.
-
-Example with 'print "Lived a boy called Outthentic"' is quite useless, here some more realistic examples:
-
-    # use of Test::More functions
-    # to modify validation workflow:
-    
-    # skip tests
-    
-    code: skip('next 3 checks are skipped',3) # skip three next checks forever
-    color: red
-    color: blue
-    color: green
-    
-    number:one
-    number:two
-    number:three
-    
-    # skip tests conditionally
-    
-    color: red
-    color: blue
-    color: green
-    
-    code: skip('numbers checks are skipped',3)  if $ENV{'skip_numbers'} # skip three next checks if skip_numbers set
-    
-    number:one
-    number:two
-    number:three
-
-Perl expressions could be effectively used with L<`captures'|#captures>:
+One of the use case for perl expressions is to store L<`captures'|#captures> data:
 
     regexp: my name is (\w+) and my age is (\d+)
-    code: cmp_ok(capture()->[1],'>',20,capture()->[0].' is adult one');
+    code: $main::data{name} = capture()->[0]; $main::data{age} = capture()->[1]; 
 
 =over
 
@@ -780,6 +1010,74 @@ Follow L<http://perldoc.perl.org/functions/eval.html|http://perldoc.perl.org/fun
 =back
 
 
+=head1 Validators
+
+=over
+
+=item *
+
+Validator expressions like perl expressions are just a piece of perl code. 
+
+
+
+=item *
+
+Validator expressions start with `validator:' marker
+
+
+
+=item *
+
+Validator code gets executed and value returned by the code is treated as validation status.
+
+
+
+=item *
+
+Validator should return array reference. First element of array is validation status and second one is helpful message which
+will be shown when status is appeared in TAP output.
+
+
+
+=back
+
+For example:
+
+    # this is always true
+    validator: [ 10>1 , 'ten is bigger then one' ]
+    
+    # and this is not
+    validator: [ 1>10, 'one is bigger then ten'  ]
+
+=over
+
+=item *
+
+Validators become very efficient when gets combined with L<`captures expressions'|#captures>
+
+
+=back
+
+This is a simple example:
+
+    # stdout
+    # it's my family ages.
+    alex    38
+    julia   25
+    jan     2
+    
+    
+    # let's capture name and age chunks
+    regexp: /(\w+)\s+(\d+)/
+    
+    validator:                          \
+    my $total=0;                        \
+    for my $c (@{captures()}) {         \
+        $total+=$c->[0];                \
+    }                                   \
+    [ ( $total == 72 ), "total age" ] 
+
+
 =head1 Generators
 
 =over
@@ -792,7 +1090,7 @@ Generators is the way to I<generate new outthentic entries on the fly>.
 
 =item *
 
-Generators like perl expressions are just a piece of perl code.
+Generator expressions like perl expressions are just a piece of perl code.
 
 
 
@@ -858,7 +1156,7 @@ Here is more complicated example:
     bar value
 
 
-=head1 multiline expressions
+=head1 Multiline expressions
 
 When generate and execute check expressions parser operates in a I<single line mode> :
 
@@ -888,18 +1186,24 @@ For example:
     regexp: Multiline \n string \n here
     
     # stdout
-    Multiline \n string \n here
+    Multiline
+    string
+    here
      
-     
-    # validation output
-    OK output matches "Multiline"
-    OK output matches "string"
-    OK output matches "here"
-    NOT_OK output matches "Multiline \n string \n here"
+     # validation results
+    
+    +--------+---------------------------------------+
+    | status | message                               |
+    +--------+---------------------------------------+
+    | OK     | matches "Multiline"                   |
+    | OK     | matches "string"                      |
+    | OK     | matches "here"                        |
+    | FAIL   | matches /Multiline \n string \n here/ |
+    +--------+---------------------------------------+
 
 Use text blocks if you want to achieve multiline checks.
 
-However when writing perl expressions or generators one could use multilines strings.
+However when writing perl expressions, validators or generators one could use multilines strings.
 
 `\' delimiters breaks a single line text on a multi lines:
 
@@ -941,12 +1245,12 @@ I<After> this regular expression check gets executed captured data will stored i
 
 Then captured data might be accessed for example by code generator to define some extra checks:
 
-    code:                               \
+    validator:                          \
     my $total=0;                        \
     for my $c (@{captures()}) {         \
         $total+=$c->[0];                \
     }                                   \
-    cmp_ok( $total,'==',72,"total age of my family" );
+    [ ($total == 72 ), "total age of my family" ];
 
 =over
 
@@ -971,12 +1275,13 @@ Here some more examples:
     # and check if it is greater then 10
     
     regexp: (\d+)
-    code:                               \
+    
+    validator:                          \
     my $total=0;                        \
     for my $c (@{captures()}) {         \
         $total+=$c->[0];                \
     }                                   \
-    cmp_ok( $total,'>',10,"total amount is greater than 10" );
+    [ ( $total > 10 ) "total amount is greater than 10" ]
     
     
     # check if stdout contains lines
@@ -984,12 +1289,14 @@ Here some more examples:
     # and then check if first date found is yesterday
     
     regexp: date: (\d\d\d\d)-(\d\d)-(\d\d)
-    code:                               \
+    
+    validator:                          \
     use DateTime;                       \
     my $c = captures()->[0];            \
     my $dt = DateTime->new( year => $c->[0], month => $c->[1], day => $c->[2]  ); \
     my $yesterday = DateTime->now->subtract( days =>  1 );                        \
-    cmp_ok( DateTime->compare($dt, $yesterday),'==',0,"first day found is - $dt and this is a yesterday" );
+    
+    [ ( DateTime->compare($dt, $yesterday) == 0 ),"first day found is - $dt and this is a yesterday" ];
 
 You also may use `capture()' function to get a I<first element> of captures array:
 
@@ -997,15 +1304,75 @@ You also may use `capture()' function to get a I<first element> of captures arra
     # a first number should be greater then ten
     
     regexp: (\d+)
-    code: cmp_ok( capture()->[0],'>',10,"first number is greater than 10" );
+    validator: [ ( capture()->[0] >  10 ), " first number is greater than 10 " ];
 
 
-=head1 AUTHOR
+=head1 Within expressions
+
+Within expression acts like regular expression but narrows search context to last matching line:
+
+    # one of 3 colors:
+    within: color: (red|green|blue)
+    
+    # if within expression is successfully passed
+    # new search context is last matching line  
+
+In other words when `:within' marker is used parser tries to validate stdout against regular expression following after :within marker and 
+if validation is successful new search context is defined:
+
+    # one of 3 colors:
+    within: color: (red|green|blue)
+    
+    # I really need a red color
+    red
+
+The code above does follows:
+
+=over
+
+=item *
+
+try to find `color:' followed by `red' or `green' or `blue' word 
+
+
+=item *
+
+if previous check is successful new context is narrowed to matching line
+
+
+=item *
+
+thus next plain string checks expression means - try to find `red' in line matching the `color: (red|green|blue)'
+
+
+=back
+
+Here more examples:
+
+    # try to find a date string in following format
+    within: date: \d\d\d\d-\d\d-\d\d
+    
+    # we only need a dates in 2000 year
+    2000-
+
+Within expressions could be sequential, which effectively means using `&&' logical operators for within expressions:
+
+    # try to find a date string in following format
+    within: date: \d\d\d\d-\d\d-\d\d
+    
+    # and try to find year of 2000 in a date string
+    within: 2000-\d\d-\d\d
+    
+    # and try to find month 04 in a date string
+    within: \d\d\d\d-04-\d\d
+
+
+=head1 Author
 
 L<Aleksei Melezhik|mailto:melezhik@gmail.com>
 
 
-=head1 Home Page
+=head1 Home page
 
 https://github.com/melezhik/outthentic-dsl
 
